global NU
global k
global h_four
global Num_drops
global Nr

%% Derivatives etc required to compute Hopf coefficients
a   = 2*pi/Num_drops;       % angular separation parameter
th0 = (1:Num_drops) * a;    % angular positions of droplets
x0  = (1:Num_drops) * LAM/Num_drops; % arc length positions of droplets

%% Define functions necessary to compute Hopf coefficients
% second, third and fourth derivative of wave kernel H at lattice points
dH2 = 1/Nr * real(  sum(-k.^2 .* h_four .* exp(1i * x0.' * k), 2)  );
dH3 = 1/Nr * real(1i*  sum(-k.^3 .* h_four .* exp(1i * x0.' * k), 2)  );
dH4 = 1/Nr * real(  sum(k.^4 .* h_four .* exp(1i * x0.' * k), 2)  );

% functions of second derivative (k is an integer)
ck = @(k) exp(-1i * k * th0) * dH2; % should be real

% functions of third derivative (k is an integer)
em1 = @(k) exp(-1i * k * th0) * dH3;
em2 = @(k) exp(-2i * k * th0) * dH3;

% functions of fourth derivative (k is an integer)
dp1 = @(k) exp(+1i * k * th0) * dH4;
d0  = sum(dH4);
dm1 = @(k) exp(-1i * k * th0) * dH4;
dm2 = @(k) exp(-2i * k * th0) * dH4;

%% Computation of Hopf coefficients
k_c  = kcrit;                  % critical wavenumber
nu_c = NU_crit;                % critical dissipation of waves
om_c = sqrt(ck(0)/nu_c + nu_c);% critical angular frequency

% dispersion relation
DR = @(lam, nu, k) lam.^2 + lam + ck(0)./nu - ck(k)./(lam + nu);

% coefficients from Hopf analysis
a1 = em1(k_c)/(nu_c + 1i * om_c) - 0.5 * em2(k_c)/(nu_c + 2i * om_c);
a2 = 2 * real(  em1(k_c)/(nu_c + 1i * om_c)  );
C1 = a1/DR(2i * om_c, nu_c, 2*k_c);
C2 = a2/(1 + ck(0)/nu_c^2);

% alpha = dD/dlambda, beta = dD/dnu, gamma  = coefficient of cubic term
alpha = 2i*om_c + 1 + ck(k_c)/(nu_c + 1i * om_c)^2;
beta  = -ck(0)/nu_c^2 + ck(k_c)/(nu_c + 1i * om_c)^2;
    
% coefficient of cubic term (so beautiful...)
gamma = 1.5 * d0/nu_c - ...
        real(dm1(k_c)/(nu_c + 1i * om_c)) + ...
        0.5 * dm2(k_c)/(nu_c + 2i * om_c) -  ...                
        dm1(k_c)/(nu_c + 1i * om_c) + ...
        2i*C1*imag( em1(k_c)/(nu_c + 1i * om_c) ) - ...
        C1 * em2(k_c)/(nu_c + 2i * om_c) - ...
        C2 * em1(k_c)/(nu_c + 1i * om_c)^2;

% normalized coefficients: beta/alpha = sig1 + 1i * om1, etc
sig1 = real(beta/alpha);    om1 = imag(beta/alpha);
sig2 = real(gamma/alpha);   om2 = imag(gamma/alpha);

% fixed point of Stuart-Landau equation (real part)
rho = sqrt(sig1/sig2);

%% Computation of key quantities
Amp_Hopf = 2 * rho * sqrt(NU_crit - NU); % real amplitude of oscillation
phis     = om1 - om2*sig1/sig2;          % frequency shift (SLE, imag part)
Om_Hopf  = om_c + phis * (NU_crit - NU); % angular frequency

drift_Hopf = abs(C2 * rho^2 * (NU_crit - NU) ); % drift speed

disp('------------------------------------------')
disp(['Hopf oscillation amplitude = ' num2str(Amp_Hopf) ])
disp(['Hopf drift speed = ' num2str(drift_Hopf) ])
disp(['Hopf angular frequency = ' num2str(Om_Hopf) ])