global k
global Nr
global hf_ind


%% plot results (dimensionless units)
% --- droplet positions ---
figure
plot(t, x_mat, 'b', 'LineWidth', 2)
set(gca, 'FontName', 'Times New Roman', 'FontSize', 16)
xlabel('$$t/T_0$$', 'Interpreter', 'Latex', 'FontSize', 20)
ylabel('$$\frac{x}{\lambda_F}$$', 'Interpreter', 'Latex', 'FontSize', 20)
set(get(gca, 'YLabel'), 'Rotation', 0)
grid on
title('Droplet positions')

% --- droplet velocities ---
figure
plot(t, v_mat, 'k', 'LineWidth', 2)
set(gca, 'FontName', 'Times New Roman', 'FontSize', 16)
xlabel('$$t/T_0$$', 'Interpreter', 'Latex', 'FontSize', 20)
ylabel('$$\frac{v T_0}{\lambda_F}$$', 'Interpreter', 'Latex', 'FontSize', 20)
set(get(gca, 'YLabel'), 'Rotation', 0)
grid on
title('Droplet velocities')

%% plot wave field at latest time
hf          = Y(hf_ind, :);

% pad out zeros for fft
h_fft       = zeros(1, Nr);
h_fft(kind) = hf;
h_plot      = real(ifft(h_fft));

% get droplet positions and wave elevations under each droplet
xd = x_mat(end,:).'; 
etad = 1/Nr * real(sum(hf.' .* exp(1i * xd * k), 2));

% plot wave field
figure
plot(x, h_plot, 'b-', 'LineWidth', 2) % free surface (dimensionless)
hold on
scatter(mod(xd, LAM), etad, 50, 'k', 'Filled') % droplet markers
hold off

axis([[0, LAM], [min(h_plot), max(h_plot)]])
set(gca, 'FontName', 'Times New Roman', 'FontSize', 16)
xlabel('$$x/\lambda_F$$', 'Interpreter', 'Latex', 'FontSize', 20)
ylabel('$$h$$', 'Interpreter', 'Latex', 'FontSize', 20)
set(get(gca, 'YLabel'), 'Rotation', 0)
grid on
title('Wave field at end of simulation')

%% Post-process amplitude, frequency and speed of oscillation
% If a periodic state is yet to be attained then ignore this output
% Compare results to Hopf_anlaysis.m
% --- drift velocity ---
x1 = x_mat(:,1); % consider first droplet only (arbitrary choice)
[pks,locs] = findpeaks(x1); 

if length(pks) > 101 
    ind = length(pks) + (-100:0); 
else
    ind = 5:length(pks);
end

pks = pks(ind); locs = locs(ind);
drift_sim = (pks(1)-pks(end))/(t(locs(1))-t(locs(end))); % drift velocity

% --- amplitude and angular frequency ---
x2 = x1 - t * drift_sim;
x2 = x2((t>(t_max - 500)));
[pks,locs] = findpeaks(x2);         % peaks of oscillation
tgh = findpeaks(-x2);  tgh = -tgh;  % troughs of oscillation
len = min(length(pks),length(tgh));
pks = pks(end-len+1:end);
tgh = tgh(end-len+1:end);
cent = (pks + tgh)/2;               % centers of oscillation

amp_sim = mean(pks-cent);           % amplitude
Om_sim = 2*pi/mean(diff(t(locs)));  % angular frequency

disp('------------------------------------------')
disp(['Simulation oscillation amplitude = ' num2str(amp_sim) ])
disp(['Simulation drift speed = ' num2str(abs(drift_sim)) ])
disp(['Simulation angular frequency = ' num2str(abs(Om_sim)) ])
